#version 330 core
out vec4 FragColor;

in vec3 TexCoords;

uniform samplerCube skybox;

// Dithering code reference: http://alex-charlton.com/posts/Dithering_on_the_GPU/
const int indexMatrix4x4[16] = int[16](
	0,  8,  2,  10,
	12, 4,  14, 6,
	3,  11, 1,  9,
	15, 7,  13, 5
);

float indexValue()
{
	int x = int(mod(gl_FragCoord.x, 4));
	int y = int(mod(gl_FragCoord.y, 4));
	return indexMatrix4x4[(x + y * 4)] / 16.0;
}

float dither(float color) {
	float closestColor = (color < 0.5) ? 0 : 1;
	float secondClosestColor = 1 - closestColor;
	float d = indexValue();
	float distance = abs(closestColor - color);
	return (distance < d) ? closestColor : secondClosestColor;
}

// Fog effect
const vec3 FOG_COLOR = vec3(0.5f, 0.5f, 0.5f);

void main() {
	// Fog effect - constant because cube walls are quite far
	float inScattering = exp(-0.8f);
	float extinction = exp(-0.8f);

	inScattering = 1.0f;
	extinction = 1.0f;

	FragColor = extinction * texture(skybox, TexCoords) + (1.0f - inScattering) * vec4(FOG_COLOR, 1.0f);
	FragColor = vec4(vec3(dither((FragColor.x + FragColor.y + FragColor.z)/3.0f) - 0.70f), 1.0f);
}