#version 330 core
in vec2 texCoord;

out vec4 fragColor;

uniform float life;
uniform sampler2D sprite;

// Dithering code reference: http://alex-charlton.com/posts/Dithering_on_the_GPU/
const int indexMatrix4x4[16] = int[16](
	0,  8,  2,  10,
	12, 4,  14, 6,
	3,  11, 1,  9,
	15, 7,  13, 5
);

float indexValue()
{
	int x = int(mod(gl_FragCoord.x, 4));
	int y = int(mod(gl_FragCoord.y, 4));
	return indexMatrix4x4[(x + y * 4)] / 16.0;
}

float dither(float color) {
	float closestColor = (color < 0.5) ? 0 : 1;
	float secondClosestColor = 1 - closestColor;
	float d = indexValue();
	float distance = abs(closestColor - color);
	return (distance < d) ? closestColor : secondClosestColor;
}

void main()
{
	vec4 textureColor = texture(sprite, texCoord);

	if (textureColor.a < 0.1)
		discard;

	textureColor.r = dither(textureColor.r);
	textureColor.g = dither(textureColor.g);
	textureColor.b = dither(textureColor.b);

	fragColor = vec4(textureColor.rgb, life);
}