#version 330 core
in vec3 fragNor;
out vec4 color;

uniform vec3 difCol;
uniform vec3 MatAmb;
uniform vec3 MatDif;
uniform vec3 MatSpec;
uniform float shine;
uniform vec3 lightCol;
uniform vec3 lightDirection;

in vec3 lightDir;
in vec3 viewDir;
in vec3 H;

// Dithering code reference: http://alex-charlton.com/posts/Dithering_on_the_GPU/
const int indexMatrix4x4[16] = int[16](
	0,  8,  2,  10,
	12, 4,  14, 6,
	3,  11, 1,  9,
	15, 7,  13, 5
);

float indexValue()
{
	int x = int(mod(gl_FragCoord.x, 4));
	int y = int(mod(gl_FragCoord.y, 4));
	return indexMatrix4x4[(x + y * 4)] / 16.0;
}

float dither(float color) {
	float closestColor = (color < 0.5) ? 0 : 1;
	float secondClosestColor = 1 - closestColor;
	float d = indexValue();
	float distance = abs(closestColor - color);
	return (distance < d) ? closestColor : secondClosestColor;
}

void main()
{
	vec3 normal = normalize(fragNor);
	vec3 light;
	vec3 result;
	light = normalize(lightDir);

	float ambientStrength = 0.5f;
	vec3 ambient = ambientStrength * MatAmb;

	float diff = max(0.0, dot(normal, light));
	vec3 diffuse = MatDif * diff;

	vec3 specular = MatSpec * max(0.0, pow(dot(normal, H), shine));

	vec3 lightAngle = lightDirection;
	float theta = dot(light, -lightAngle);

	result += (ambient + diffuse + specular) * (theta / 2.0);
	result = vec3(dither((result.x + result.y + result.z)/3.0f) - 0.4f);

	color = vec4(result, 1.0);
}
