#version 330 core
uniform sampler2D Texture0;
uniform int texID;
uniform vec3 lightDirection;

in vec2 vTexCoord;
in vec3 fragNor;
in vec3 lightDir;
in vec3 H;
out vec4 Outcolor;

// Dithering code reference: http://alex-charlton.com/posts/Dithering_on_the_GPU/
const int indexMatrix4x4[16] = int[16](
	0,  8,  2,  10,
	12, 4,  14, 6,
	3,  11, 1,  9,
	15, 7,  13, 5
);

float indexValue()
{
	int x = int(mod(gl_FragCoord.x, 4));
	int y = int(mod(gl_FragCoord.y, 4));
	return indexMatrix4x4[(x + y * 4)] / 16.0;
}

float dither(float color) {
	float closestColor = (color < 0.5) ? 0 : 1;
	float secondClosestColor = 1 - closestColor;
	float d = indexValue();
	float distance = abs(closestColor - color);
	return (distance < d) ? closestColor : secondClosestColor;
}

void main() {
	vec3 normal = normalize(fragNor);
	vec3 light;
	vec3 result;

	light = normalize(lightDir);

	float ambientStrength = 0.5f;
	vec4 texColor0 = texture(Texture0, vTexCoord);

	vec4 ambient = texColor0 * ambientStrength;

	float diff = max(0.0, dot(normal, light));
	vec4 diffuse = texColor0 * diff;

	vec4 specular = vec4(vec3(texColor0.xyz * max(0.0, pow(dot(normal, H), 80.0))), 1.0);

	vec3 lightAngle = lightDirection;
	float theta = dot(light, -lightAngle);

	result += (ambient + diffuse + specular).xyz * (theta / 2.0);
	result = vec3(dither((result.x + result.y + result.z)/3.0f) - 0.45f);

	Outcolor = vec4(result, 1.0);
}

