#version 330 core

out vec4 color;

uniform vec3 MatAmb;
uniform vec3 MatDif;
uniform vec3 MatSpec;
uniform float shine;

uniform sampler2D depthTexture;

in VS_OUT {
	float worldHeight;
	vec3 lightDir;
	vec3 H;
	vec3 fragNor;
	vec4 fragPosFromLightPOV;
} fs_in;

// in GS_OUT {
// 	vec3 fragNor;
// } gs_to_fs;

// Dithering code reference: http://alex-charlton.com/posts/Dithering_on_the_GPU/
const int indexMatrix4x4[16] = int[16](
	0,  8,  2,  10,
	12, 4,  14, 6,
	3,  11, 1,  9,
	15, 7,  13, 5
);

float indexValue()
{
	int x = int(mod(gl_FragCoord.x, 4));
	int y = int(mod(gl_FragCoord.y, 4));
	return indexMatrix4x4[(x + y * 4)] / 16.0;
}

float dither(float color) {
	float closestColor = (color < 0.5) ? 0 : 1;
	float secondClosestColor = 1 - closestColor;
	float d = indexValue();
	float distance = abs(closestColor - color);
	return (distance < d) ? closestColor : secondClosestColor;
}

float testShadow()
{
	// Apparently this solves shadow acne
	// float bias = max(0.05f * (1.0f - dot(normalize(fs_in.fragNor), normalize(fs_in.lightDir))), 0.005f);
	float bias = 0.0f; 	// This is not playing nice atm
	// Because depth texture, or any texture, ranges from [0, 1]
	//  we need to map our NDC from [-1, 1] to [0, 1]
	vec3 mappedFragPosFromLightPOV = 0.5f * fs_in.fragPosFromLightPOV.xyz + 0.5f;
	// Sample depth texture. Depth information stored in r value
	// Percentage-closer filtering
	float shadow = 0.0f;

	if (mappedFragPosFromLightPOV.z > 1.0f) {
		shadow = 0.0;
	} else {
		vec2 texelSize = 1.0f / textureSize(depthTexture, 0);
		for(int x = -1; x <= 1; ++x)
		{
			for(int y = -1; y <= 1; ++y)
			{
				float pcfDepth = texture(depthTexture, mappedFragPosFromLightPOV.xy + vec2(x, y) * texelSize).r;
				shadow += mappedFragPosFromLightPOV.z - bias > pcfDepth ? 0.5f : 0.0f;
			}
		}
		shadow /= 9.0f;
	}
	return shadow;
}

void main()
{
	vec3 normal = normalize(fs_in.fragNor);
	vec3 light = normalize(fs_in.lightDir);
	vec3 result;

	vec3 diffuse = MatDif * max(0.0, dot(normal, light));
	vec3 specular = MatSpec * max(0.0, pow(dot(normal, fs_in.H), shine));

	result = 0.1f * MatAmb + 0.3f * diffuse + 0.3f * specular;

	float heightCol = ((fs_in.worldHeight + 0.9f) / 1.8f);
	result += heightCol * vec3(0.2f);

	result = vec3(dither((result.x + result.y + result.z) / 3.0f) - 0.20f);

	color = vec4((1.0f - testShadow()) * result, 1.0f);
}