#version 330 core
#extension GL_EXT_gpu_shader4: enable
#extension GL_EXT_geometry_shader4: enable

precision highp float;

layout (triangles_adjacency) in;
layout (triangle_strip, max_vertices = 3) out;

uniform mat4 M;

in VS_OUT {
	float worldHeight;
	vec3 lightDir;
	vec3 H;
} gs_in[];

out VS_OUT {
	float worldHeight;
	vec3 lightDir;
	vec3 H;
} vs_out;

out GS_OUT {
	vec3 fragNor;
} gs_out;

vec3 computeNormal(vec3 v0, vec3 v1, vec3 v2)
{
	vec3 firstEdge = v0 - v1;
	vec3 secondEdge = v2 - v1;
	return normalize(cross(firstEdge, secondEdge));
}

vec3 averageNormals(vec3 n0, vec3 n1, vec3 n2)
{
	return normalize((n0 + n1 + n2) / 3);
}

// Reference: https://web.engr.oregonstate.edu/~mjb/cs557/Handouts/geometry_shaders.1pp.pdf
void main()
{
	vec3 v0 = gl_PositionIn[0].xyz;
	vec3 v1 = gl_PositionIn[1].xyz;
	vec3 v2 = gl_PositionIn[2].xyz;
	vec3 v3 = gl_PositionIn[3].xyz;
	vec3 v4 = gl_PositionIn[4].xyz;
	vec3 v5 = gl_PositionIn[5].xyz;

	vec3 n042 = computeNormal(v4, v0, v2);
	vec3 n021 = computeNormal(v2, v0, v1);
	vec3 n243 = computeNormal(v4, v2, v3);
	vec3 n405 = computeNormal(v0, v4, v5);

	uint j = 0u;
	for (uint i = 0u; i < 3u; ++i) {
		// j = i * 2u;
		j = i;
		gl_Position = gl_PositionIn[j];
		gs_out.fragNor = vec3(0.0f, 1.0f, 0.0f);

		vs_out.worldHeight = gs_in[j].worldHeight;
		vs_out.lightDir = gs_in[j].lightDir;
		vs_out.H = gs_in[j].H;

		EmitVertex();
	}

	EndPrimitive();
}