#version  330 core
layout(location = 0) in vec4 vertPos;
layout(location = 1) in vec3 vertNor;

uniform mat4 P;
uniform mat4 V;
uniform mat4 M;

const float PI = 3.1415926535897932384626433832795;

const float xOffset = 0.007812f;
const float zOffset = 0.007812f;

uniform vec3 eyePos;
uniform vec3 lightPos;
uniform float time;
uniform float waveLen;
uniform float waveAmp;

uniform mat4 LS;

out VS_OUT {
	float worldHeight;
	vec3 lightDir;
	vec3 H;
	vec3 fragNor;
	vec4 fragPosFromLightPOV;
} vs_out;

// Mathematic credit: https://github.com/TheThinMatrix/LowPolyWater
float offset(float x, float z)
{
	z += x;

	float radiansX = ((0.5f * x) / waveLen + (0.25f * time)) * 2.0f * PI;
	float radiansZ = ((0.5f * z) / waveLen + (0.25f * time)) * 2.0f * PI;

	return waveAmp * 0.5 * (sin(radiansZ) + cos(radiansX));
}

vec3 computeNormal(vec3 currentPosition)
{
	// Offset x and z by 0.007812 unit depending on direction.
	//  Generate 4 points => 4 faces
	vec3 north, south, west, east;
	vec3 normalNW, normalNE, normalSW, normalSE;
	vec3 edge_1, edge_2;

	north.x = currentPosition.x;
	north.z = currentPosition.z - zOffset;
	north.y = offset(north.x, north.z);

	south.x = currentPosition.x;
	south.z = currentPosition.z + zOffset;
	south.y = offset(south.x, south.z);

	west.x = currentPosition.x - xOffset;
	west.z = currentPosition.z;
	west.y = offset(west.x, west.z);

	east.x = currentPosition.x + xOffset;
	east.z = currentPosition.z;
	east.y = offset(east.x, east.z);

	// Compute normals of each generated face.
	// North-west face
	edge_1 = north - currentPosition;
	edge_2 = west - currentPosition;
	normalNW = cross(edge_1, edge_2);


	// North-east face
	edge_2 = east - currentPosition;
	normalNE = cross(edge_2, edge_1);

	// South-west face
	edge_1 = south - currentPosition;
	edge_2 = west - currentPosition;
	normalSW = cross(edge_2, edge_1);


	// South-east face
	edge_1 = south - currentPosition;
	normalSE = cross(edge_1, edge_2);

	// Sum and normalize all the normals
	return vec3(normalize(normalNW + normalNE + normalSW + normalSE));
}

void main()
{
	vec4 newVertPos = vertPos;

	float x = newVertPos.x;
	float z = newVertPos.z;

	newVertPos.y = offset(x, z);
	vs_out.worldHeight = newVertPos.y;	// Map this to world space

	gl_Position = P * V * M * vec4(newVertPos.xyz, 1.0f);

	vs_out.lightDir = lightPos - (M * newVertPos).xyz;
	vec3 viewDir = normalize(eyePos - newVertPos.xyz);
	vs_out.H = normalize(vs_out.lightDir + viewDir);

	vs_out.fragNor = (M * vec4(computeNormal(newVertPos.xyz), 0.0f)).xyz;
	vs_out.fragPosFromLightPOV = LS * M * vec4(newVertPos.xyz, 1.0f);
}